/*
 * Xnotepad: Simple text editor.
 *
 * $Log: $
 *
 *
 */

#ifndef lint
static char        *rcsid = "$Id: $";
#endif

/* System stuff */
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>

/* Include all the widget stuff we need */
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <Xpw/Clue.h>
#include <Xpw/RowCol.h>
#include <Xpw/PopupMenu.h>
#include <Xpw/MenuBar.h>
#include <Xpw/BarButton.h>
#include <Xpw/PmeEntry.h>
#include <Xpw/PmeLine.h>
#include <Xpw/Label.h>
#include <Xpw/Command.h>
#include <Xpw/Dialog.h>
#include <Xpw/SimpleText.h>

#if 0
typedef struct _XNotePadResources {
    int		foo;
} XNotePadResources;

static XrmOptionDescRec options[] =
{
};

#define Offset(field) (XtOffsetOf(XNotePadResources, field))

static XtResource   my_resources[] =
{
    {"notepadDevice", XtCString, XtRString, sizeof(String),
     Offset(notepad_device), XtRImmediate, (XtPointer) "/dev/notepad"},
    {"showAll", XtCBoolean, XtRBoolean, sizeof(Boolean),
     Offset(show_all), XtRImmediate, (XtPointer) FALSE},
    {"pollRate", "PollRate", XtRInt, sizeof(int),
     Offset(poll_rate), XtRImmediate, (XtPointer) 500},
};

#undef Offset

static XNotePadResources resources;
#endif

static void         usage(void);
void                main(int /*argc */ , char ** /*argv */ );
static Pixmap	    MakePixmap(char **/*data*/, char */*data_bit*/,
			 int /*width*/, int /*height*/, Pixmap */*mask*/);
static void         quit(Widget /*w */ , XEvent * /*event */ ,
			 String * /*params */ , Cardinal * /*num_params */ );
static void         app_quit(Widget /*w */ , XtPointer /*client_data */ ,
			 XtPointer /*call_data */ );
static void	    iconify(Widget /*w */, XtPointer /*client_data */,
			 XEvent */*event*/, Boolean */*dispatch*/);
static void         error_dialog(String /*message */);
static void         notice_dialog(String /*message */);

/*
 * Define the bare minimum here to make appliation look correct.
 */
static String       fallback_resources[] =
{
   /* Global Stuff */
    "XNotePad*Frame.outline:	   false",
   /* Labels */
    "XNotePad*botbar*Slider.width:   20",
    "XNotePad*left.label: 	   Left",
    "XNotePad*right.label:	   Right",
    "XNotePad*loud.label:		   Loudness",
    "XNotePad*switch.shadowWidth:	   0",
    "XNotePad*holder.shadowWidth:	   0",
    "XNotePad*switch.switchSize:	   20",
    "XNotePad*switch.switchShape:	   square",
    "XNotePad*menubar.width:		   200",
    "XNotePad*editArea.width:		   200",
    "XNotePad*editArea.height:		   200",
    NULL
};

static XtActionsRec xnotepad_actions[] =
{
    {"quit", quit},
};

static XtAppContext        app_con;
static Atom                wm_delete_window;
static char               *ProgramName;
static Display            *dpy;
static Window	           win;
static Widget		   toplevel;
static Widget		   dialog = NULL;

static void 
usage()
{
    fprintf(stderr, "usage:  %s [-options ...]\n\n", ProgramName);
    fprintf(stderr, "where options include:\n");
    fprintf(stderr,
	    "    -display dpy            X server on which to display\n");
    fprintf(stderr,
	    "    -geometry geom          size and location of window\n");
    fprintf(stderr,
	    "    -fn font                font to use in label\n");
    fprintf(stderr,
	    "    -bg color               background color\n");
    fprintf(stderr,
	    "    -fg color               foreground color\n");
    exit(0);
}

void 
main(argc, argv)
	int                 argc;
	char              **argv;
{
    Widget              manager, menu, w, w1;
    Arg                 args[5];
    Pixmap              image, mask;
    int                 i;
    char		ebuffer[200];

    ProgramName = argv[0];

#if 0
    toplevel = XtAppInitialize(&app_con, "XNotePad", options, XtNumber(options),
		    &argc, argv, fallback_resources, NULL, (Cardinal) 0);
#endif
    toplevel = XtAppInitialize(&app_con, "XNotePad", NULL, (Cardinal) 0,
		    &argc, argv, fallback_resources, NULL, (Cardinal) 0);
    if (argc != 1)
	usage();

#if 0
    XtGetApplicationResources(toplevel, (XtPointer) & resources,
			      my_resources, XtNumber(my_resources),
			      NULL, (Cardinal) 0);
#endif
    XtAppAddActions(app_con, xnotepad_actions, XtNumber(xnotepad_actions));

    dpy = XtDisplay(toplevel);
    win = DefaultRootWindow(dpy);

   /* Add Clue widget */
    XtCreatePopupShell("CluePopup", clueWidgetClass, toplevel, NULL, 0);

   /* Build widgets. */
    manager = XtCreateManagedWidget("manager", rowColWidgetClass, toplevel,
			NULL, 0);

    menu = XtCreateManagedWidget("menubar", menubarWidgetClass, manager,
                        NULL, 0);
    /* File menu */
    XtSetArg(args[0], XtNmenuName, "filemenu");
    w = XtCreateManagedWidget("file", barbuttonWidgetClass, menu, args, 1);
    w = XtCreatePopupShell("filemenu", popupMenuWidgetClass, menu, NULL, 0);
    w1 = XtCreateManagedWidget("new", pmeEntryObjectClass, w, NULL, 0);
    w1 = XtCreateManagedWidget("open", pmeEntryObjectClass, w, args, 1);
    w1 = XtCreateManagedWidget("save", pmeEntryObjectClass, w, NULL, 0);
    w1 = XtCreateManagedWidget("saveas", pmeEntryObjectClass, w, NULL, 0);
    XtCreateManagedWidget("mline", pmeLineObjectClass, w, NULL, 0);
    w1 = XtCreateManagedWidget("options", pmeEntryObjectClass, w, NULL, 0);
#if 0
    XtAddCallback(w1, XtNcallback, options, (XtPointer)this);
#endif
    w1 = XtCreateManagedWidget("exit", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);

    /* Display control menu */
    XtSetArg(args[0], XtNmenuName, "editmenu");
    w = XtCreateManagedWidget("edit", barbuttonWidgetClass, menu, args, 1);
    w = XtCreatePopupShell("editmenu", popupMenuWidgetClass, menu, NULL, 0);
    w1 = XtCreateManagedWidget("cut", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);
    w1 = XtCreateManagedWidget("copy", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);
    w1 = XtCreateManagedWidget("paste", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);
    w1 = XtCreateManagedWidget("search", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);
    w1 = XtCreateManagedWidget("replace", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);
    w1 = XtCreateManagedWidget("include", pmeEntryObjectClass, w, NULL, 0);
    XtAddCallback(w1, XtNcallback, app_quit, NULL);

    XtSetArg(args[0], XtNallowResize, TRUE);
    XtSetArg(args[1], XtNresizeToPreferred, FALSE);
    w1 = XtCreateManagedWidget( "editArea", simpleTextWidgetClass, manager,
			 args, 2);

    XtRealizeWidget(toplevel);

   /* This is a hack so that f.delete will do something useful. */
    XtOverrideTranslations(toplevel,
	       XtParseTranslationTable("<Message>WM_PROTOCOLS: quit()"));

    wm_delete_window = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
    (void) XSetWMProtocols(dpy, XtWindow(toplevel), &wm_delete_window, 1);

    XtAppMainLoop(app_con);
}

/*
 * Handle delete correctly.
 */
static void 
quit(w, event, params, num_params)
	Widget              w;
	XEvent             *event;
	String             *params;
	Cardinal           *num_params;
{
    if (event->type == ClientMessage &&
	event->xclient.data.l[0] != wm_delete_window) {
	XBell(dpy, 0);
	return;
    }
    XtDestroyApplicationContext(app_con);
    exit(0);
}

/* 
 * Quit button call back.
 */
static void 
app_quit(w, client_data, call_data)
	Widget              w;
	XtPointer           client_data;
	XtPointer           call_data;
{
    XtDestroyApplicationContext(app_con);
    exit(0);
}

/*
 * Pop up a error dialog, quit on Ok.
 */
static void
error_dialog(message)
	String		   message;
{
        Arg     arg[1];
   
        XtSetArg(arg[0], XtNmessage, message);
        dialog = XpwDialogCreateError(toplevel, "Xnotepad", arg, 1);
        XtUnmanageChild(XpwDialogGetChild(dialog, XpwDialog_Cancel_Button));
        XtUnmanageChild(XpwDialogGetChild(dialog, XpwDialog_Help_Button));
        XtAddCallback(XpwDialogGetChild(dialog, XpwDialog_Ok_Button),
		XtNcallback, app_quit, toplevel);
        XtManageChild(dialog);
}

/*
 * Dialog popdown.
 */
static void 
dialog_done(w, client_data, call_data)
	Widget              w;
	XtPointer           client_data;
	XtPointer           call_data;
{
	dialog = NULL;
}


/*
 * Pop up a error dialog, quit on Ok.
 */
static void
notice_dialog(message)
	String		   message;
{
        Arg     arg[1];

	if (dialog != NULL)	/* Don't flood user with dialogs */
		return;
        XtSetArg(arg[0], XtNmessage, message);
        dialog = XpwDialogCreateNotice(toplevel, "Xnotepad", arg, 1);
        XtUnmanageChild(XpwDialogGetChild(dialog, XpwDialog_Cancel_Button));
        XtUnmanageChild(XpwDialogGetChild(dialog, XpwDialog_Help_Button));
        XtAddCallback(XpwDialogGetChild(dialog, XpwDialog_Ok_Button),
		XtNcallback, dialog_done, toplevel);
        XtManageChild(dialog);
}
