/* pppmon/src/xpppmon.c
 *
 * Description:
 * Connect to the pppmon server, display statistics with a simple
 * curve (solid or a line).
 *
 * This software is in the public domain.
 *
 * Please send all bug reports, requests for information, etc. to:
 *   Calle Karlsson
 *   KaSH Software AB
 *   calle@kash.se
 *
 * History:
 * converted to Xt and Xpw widget 
 * 950525 ckn Initial coding - version 0.1
 * EndHistory.
 */

#include <stdio.h>
#include "pppmon.h"
#include <ctype.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <Xpw/Cardinals.h>
#include <Xpw/Chart.h>
#include <X11/Xmu/SysUtil.h>

#include "pppmon.bit"
/*
 * Definition of the Application resources structure.
 */

typedef struct _XPPPmonResources {
    String              hostname;
    int                 bps;
} XPPPmonResources;

/*
 * Command line options table.  Only resources are entered here...there is a
 * pass over the remaining options after XtParseCommand is let loose. 
 */

static XrmOptionDescRec options[] =
{
    {"-scale", "*pppmon.minScale", XrmoptionSepArg, NULL},
    {"-update", "*pppmon.interval", XrmoptionSepArg, NULL},
    {"-hl", "*pppmon.scalecolor", XrmoptionSepArg, NULL},
    {"-highlight", "*pppmon.scalecolor", XrmoptionSepArg, NULL},
    {"-jumpscroll", "*pppmon.scrollAmount", XrmoptionSepArg, NULL},
    {"-host", "*hostname", XrmoptionSepArg, "localhost"},
    {"-bps", "*bps", XrmoptionSepArg, (XtPointer)14400},
};

/*
 * The structure containing the resource information for the
 * Xpppmon application resources.
 */

#define Offset(field) (XtOffsetOf(XPPPmonResources, field))

static XtResource   my_resources[] =
{
    {"hostname", XtCString, XtRString, sizeof(String),
     Offset(hostname), XtRImmediate, (XtPointer) "localhost"},
    {"bps", "Bps", XtRInt, sizeof(int),
     Offset(bps), XtRImmediate, (XtPointer) 14400},
};

#undef Offset

static XPPPmonResources resources;

static void quit(Widget /*w*/, XEvent */*event*/, String */*params*/, Cardinal */*num_params*/);
static void CheckPPP(XtPointer /*closure*/, int */*source*/, XtInputId */*id*/);
static int ConnectTo(char */*HostName*/, char */*Service*/, char */*Protocol*/);

static XtActionsRec xpppmon_actions[] =
{
    {"quit", quit},
};
static Atom         wm_delete_window;

/*
 * Exit with message describing command line format.
 */

static void
                    usage()

{
    printf("Usage: xmlmon [options] where options is one or more of:\n");
    printf(" -bps <bps>      Give baudrate of modem (default 14400)\n");
    printf(" -solid          Draw a solid curve\n");
    printf(" -help           Display generic XView help\n");
    printf(" -h              You already know about this option\n");
    printf(" -host <host>    Connect to <hostname> for statistics\n");
    exit(1);
}

int
main(int argc, char **argv)
{
    char               *ip = NULL, Buffer[64];
    XtAppContext        app_con;
    Widget              toplevel, chart, pane, chart_parent;
    Arg                 args[3];
    Pixmap              icon_pixmap = None;
    int                 ListenFD;


    toplevel = XtAppInitialize(&app_con, "XPPPmon", options, XtNumber(options),
			       &argc, argv, NULL, NULL, (Cardinal) 0);
    if (argc != 1)
	usage();

    XtGetApplicationResources(toplevel, (XtPointer) & resources,
			      my_resources, XtNumber(my_resources),
			      NULL, (Cardinal) 0);


   /*
    * This is a hack so that f.delete will do something useful in this
    * single-window application.
    */
    XtAppAddActions(app_con, xpppmon_actions, XtNumber(xpppmon_actions));
    XtOverrideTranslations(toplevel,
	       XtParseTranslationTable("<Message>WM_PROTOCOLS: quit()"));

    XtSetArg(args[0], XtNiconPixmap, &icon_pixmap);
    XtGetValues(toplevel, args, ONE);
    if (icon_pixmap == None) {
	XtSetArg(args[0], XtNiconPixmap,
		 XCreateBitmapFromData(XtDisplay(toplevel),
				       XtScreen(toplevel)->root,
				       (char *) pppmon_bits,
				       pppmon_width, pppmon_height));
	XtSetValues(toplevel, args, ONE);
    }
    XtSetArg(args[0], XtNlabel, "Starting");
    XtSetArg(args[1], XtNinterval, 0);
    chart = XtCreateManagedWidget("pppmon", chartWidgetClass,
				  toplevel, args, TWO);

    XtRealizeWidget(toplevel);
    wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW",
				   False);
    (void) XSetWMProtocols(XtDisplay(toplevel), XtWindow(toplevel),
			   &wm_delete_window, 1);

    if ((ListenFD = ConnectTo(resources.hostname, "pppmon", "tcp")) < 0) {
	printf("Couldn't connect to %s\n", resources.hostname);
	exit(1);
    }
    XtAppAddInput(app_con, ListenFD, (XtPointer)XtInputReadMask, CheckPPP, chart);
    XtAppMainLoop(app_con);
    return 0;
}

static void 
quit(w, event, params, num_params)
	Widget              w;
	XEvent             *event;
	String             *params;
	Cardinal           *num_params;
{
    if (event->type == ClientMessage &&
	event->xclient.data.l[0] != wm_delete_window) {
	XBell(XtDisplay(w), 0);
	return;
    }
    XtDestroyApplicationContext(XtWidgetToApplicationContext(w));
    exit(0);
}

/* Called when we get a message from our statistics provider */
static              void
CheckPPP(closure, source, id)
	XtPointer           closure;
	int                *source;
	XtInputId          *id;
{
    static int          LastWasDown = 0;
    Widget              chart = (Widget) closure;
    int                 ListenFD = *source;
    char                Buffer[32];
    Arg			arg[1];
    DclPPPMonitorMsg    PMP;

    if (read(ListenFD, &PMP, sizeof(PMP)) != sizeof(PMP)) {
	XtSetArg(arg[0], XtNlabel, "Link Error ");
	XtSetValues(chart, arg, ONE);
	XpwChartAddData(chart, (double) 0);
	LastWasDown = 0;
	return;
    }
    PMP.UpTime = ntohl(PMP.UpTime);
    PMP.RBytes = ntohl(PMP.RBytes);
    PMP.SBytes = ntohl(PMP.SBytes);
    PMP.TimeSpent = ntohl(PMP.TimeSpent);
    PMP.RTT = ntohl(PMP.RTT);	/* NIY */

    if (!PMP.UpTime) {
	if (!LastWasDown) {
	    LastWasDown = 1;
	    XtSetArg(arg[0], XtNlabel, "Link DOWN  ");
	    XtSetValues(chart, arg, ONE);
        }
	XpwChartAddData(chart, (double) 0);
	return;
    }
    LastWasDown = 0;

    XpwChartAddData(chart, (float)((PMP.RBytes+PMP.SBytes)) /
				 (float) PMP.TimeSpent);

    if (PMP.UpTime > 3600)
	sprintf(Buffer, "UP %4dh%2dm",
		PMP.UpTime / 3600, (PMP.UpTime % 3600) / 60);
    else if (PMP.UpTime > 59)
	sprintf(Buffer, "UP %2dm%2ds", PMP.UpTime / 60, PMP.UpTime % 60);
    else
	sprintf(Buffer, "UP %2ds   ", PMP.UpTime);

    XtSetArg(arg[0], XtNlabel, Buffer);
    XtSetValues(chart, arg, ONE);
}

/* Connect to our statistics provider */

static int
ConnectTo(char *HostName, char *Service, char *Protocol)
{
    struct servent     *sp;
    struct hostent     *hp;
    struct sockaddr_in  Addr[1];
    int                 fd, Port;

    if (!(sp = getservbyname(Service, Protocol))) {
	perror("getservbyname");
	Port = 5676;		/* Any port will do as long as it's the same as pppmon.c */
    } else
	Port = sp->s_port;

    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	perror("socket");
	return -1;
    }
    if (hp = gethostbyname(HostName))
	memcpy(&Addr->sin_addr, hp->h_addr, hp->h_length);
    else {
	unsigned long       f;

	if ((f = inet_addr(HostName)) < 0) {
	    printf("Unknown host (%s)\n", HostName);
	    return (-1);
	}
	memcpy(&Addr->sin_addr, &f, sizeof(f));
    }
    Addr->sin_family = AF_INET;
    Addr->sin_port = ntohs(Port);

    if (connect(fd, (struct sockaddr *) Addr, sizeof(Addr)) < 0) {
	perror("connect");
	return (-1);
    }
    return fd;
}
